/*  kglcontrol.cpp - 
    This file is part of KGL
    Copyright (C) 2000  Antonio Larrosa Jimenez
    KGL' homepage : http://www.arrakis.es/~rlarrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kglcontrol.h" 
#include "kgluniverse.h"
#include <klocale.h>
#include <qscrollbar.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qtooltip.h>

KGLControls::KGLControls(QWidget *parent, const char *name) : QVBox( parent, name)
{
  QHBox *hbox=new QHBox(this);
  m_sliderX= new QSlider(-20000,20000,1,0,Qt::Vertical,hbox,"scrollbarx");
  QToolTip::add(m_sliderX, "X" );
  m_sliderY= new QSlider(-20000,20000,1,0,Qt::Vertical,hbox,"scrollbary");
  QToolTip::add(m_sliderY, "Y" );
  m_sliderZ= new QSlider(-20000,20000,1,0,Qt::Vertical,hbox,"sliderZ");
  QToolTip::add(m_sliderZ, "Z" );
  m_sliderPhi= new QSlider(-180000,180000,1,0,Qt::Vertical,hbox,"sliderPhi");
  QToolTip::add(m_sliderPhi, i18n("Phi") );
  m_sliderPsi= new QSlider(-90000,90000,1,0,Qt::Vertical,hbox,"sliderPsi");
  QToolTip::add(m_sliderPsi, i18n("Psi") );
  m_sliderTwist= new QSlider(-180000,180000,1,0,Qt::Vertical,hbox,"sliderTwist");
  QToolTip::add(m_sliderTwist, i18n("Twist") );
  m_sliderDistanceToCenter= new QSlider(0,10000,1,0,Qt::Vertical,hbox,"sliderDistanceToCenter");
  QToolTip::add(m_sliderDistanceToCenter, i18n("Distance to Center of Rotation Sphere") );
  m_sliderViewAngle= new QSlider(0,90000,1,0,Qt::Vertical,hbox,"sliderViewAngle");
  QToolTip::add(m_sliderViewAngle, i18n("View Angle") );
  m_moveForward=new QPushButton(i18n("Forward"),this);
  m_moveForward->setAutoResize(TRUE);
  m_moveBackward=new QPushButton(i18n("Backward"),this);
  m_moveBackward->setAutoResize(TRUE);
  m_lookToOrigin=new QPushButton(i18n("Look to Origin"),this);
  m_lookToOrigin->setAutoResize(TRUE);
}


KGLControls::~KGLControls()
{

}

void KGLControls::plug(KGLUniverse *u)
{
  m_universe=u;

  connect(m_sliderX,SIGNAL(valueChanged(int)),m_universe,SLOT(moveX(int)));
  connect(m_sliderY,SIGNAL(valueChanged(int)),m_universe,SLOT(moveY(int)));
  connect(m_sliderZ,SIGNAL(valueChanged(int)),m_universe,SLOT(moveZ(int)));
  connect(m_sliderPhi,SIGNAL(valueChanged(int)),m_universe,SLOT(movePhi(int)));
  connect(m_sliderPsi,SIGNAL(valueChanged(int)),m_universe,SLOT(movePsi(int)));
  connect(m_sliderTwist,SIGNAL(valueChanged(int)),m_universe,SLOT(twist(int)));
  connect(m_sliderTwist,SIGNAL(sliderReleased()),m_universe,SLOT(resetTwist()));
  connect(m_sliderDistanceToCenter,SIGNAL(valueChanged(int)),m_universe,SLOT(setDistanceToCenter(int)));
  connect(m_sliderViewAngle,SIGNAL(valueChanged(int)),m_universe,SLOT(setViewAngle(int)));

  
  connect(m_moveForward,SIGNAL(clicked()),m_universe,SLOT(moveForward()));
  connect(m_moveBackward,SIGNAL(clicked()),m_universe,SLOT(moveBackward()));
  connect(m_lookToOrigin,SIGNAL(clicked()),m_universe,SLOT(lookToOrigin()));

  connect(m_universe, SIGNAL(movedX(int)), m_sliderX,SLOT(setValue(int)));
  connect(m_universe, SIGNAL(movedY(int)), m_sliderY,SLOT(setValue(int)));
  connect(m_universe, SIGNAL(movedZ(int)), m_sliderZ,SLOT(setValue(int)));
  connect(m_universe, SIGNAL(phiChanged(int)), m_sliderPhi,SLOT(setValue(int)));
  connect(m_universe, SIGNAL(psiChanged(int)), m_sliderPsi,SLOT(setValue(int)));
  connect(m_universe, SIGNAL(twistChanged(int)), m_sliderTwist,SLOT(setValue(int)));
  connect(m_universe, SIGNAL(distanceToCenterChanged(int)), m_sliderDistanceToCenter,SLOT(setValue(int)));

  m_universe->moveViewPoint(15,6,4);
  m_universe->lookTo(0,0,0);
  m_sliderViewAngle->setValue(40000);
}
  
