/*  kgldataanimate.cpp - 
    This file is part of KGL
    Copyright (C) 2000  Antonio Larrosa Jimenez
    KGL' homepage : http://www.arrakis.es/~rlarrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kgldataanimate.h"
#include "kglsphere.h"
#include <kdebug.h>
#include <qtimer.h>
#include <stdlib.h>

KGLDataAnimate::KGLDataAnimate(QWidget *parent,const char *name) 
  : KGLUniverse(parent,name)
{
  m_ntraces=0;
  for (int i=0;i<8;i++)
  {
    m_dataX[i]=0L;
    m_dataY[i]=0L;
    m_dataZ[i]=0L;
    m_count[i]=0;
	  
    m_trace[i]=0L;

    m_color[i]=Qt::red; 
  }
  
  m_timer=new QTimer(this);
  connect(m_timer,SIGNAL(timeout()), this, SLOT(timerTriggered()));
}

KGLDataAnimate::~KGLDataAnimate()
{
  kdDebug() << "KGLDA::destructor" << endl;
  m_timer->stop();
  delete m_timer;
  
  for (int i=0;i<m_ntraces;i++) 
  {
    delete [] m_dataX[i];
    delete [] m_dataY[i];
    delete [] m_dataZ[i];

    objects->remove(m_trace[i]);
    delete m_trace[i];
    m_trace[i]=0L;

  }

}

int KGLDataAnimate::setData(double *x,double *y,double *z,int count)
{
  kdDebug() << "XX" << endl;
  int trace=m_ntraces;
  m_count[trace]=count;
  m_dataX[trace]=new double[count];
  m_dataY[trace]=new double[count];
  m_dataZ[trace]=new double[count];
  m_animCount[trace]=0;

  memcpy((char*)m_dataX[trace],(char*)x,count*sizeof(double));
  memcpy((char*)m_dataY[trace],(char*)y,count*sizeof(double));
  memcpy((char*)m_dataZ[trace],(char*)z,count*sizeof(double));
  kdDebug() << "YY" << endl;

  KGLObject::color(m_color[trace],0);
  kdDebug() << "YY1" << endl;
  kdDebug() << x[0] << " " << y[0] << " " << z[0] << "\n" << endl;
  if ( m_isInitialized )
  {
    m_trace[trace]=new KGLSphere(x[0],y[0],z[0],0.2,24);


    objects->append(m_trace[trace]);
  }

  kdDebug() << "YY2" << endl;

  kdDebug() << "ZZ" << endl;
  ++m_ntraces;

  if (!m_timer->isActive()) m_timer->start(150);
  kdDebug() << endl;

  return trace;
}

void KGLDataAnimate::setPlotColor(int idx, const QColor &color)
{
  m_trace[idx]->setColor(color);
}

void KGLDataAnimate::timerTriggered()
{
    for (int i=0;i<m_ntraces;i++)
    {
      m_animCount[i]++;
      
      if (m_animCount[i]==m_count[i]) m_animCount[i]=0;
      
      if ( m_trace[i]==0L && m_isInitialized )
      {
	m_trace[i]=new KGLSphere(m_dataX[i][m_animCount[i]],
	    m_dataY[i][m_animCount[i]],m_dataZ[i][m_animCount[i]],0.2,24);
	objects->append(m_trace[i]);
      }
      else 
      {
	m_trace[i]->moveTo(m_dataX[i][m_animCount[i]],
	    m_dataY[i][m_animCount[i]],m_dataZ[i][m_animCount[i]]); 
      }

/*      kdDebug() << m_dataX[i][m_animCount[i]] << " " << m_dataY[i][m_animCount[i]] << " " << m_dataZ[i][m_animCount[i]] << "\n" << endl;
*/
//      m_trace[i]->setColor(QColor(rand()*255.0/RAND_MAX,rand()*255.0/RAND_MAX,rand()*255.0/RAND_MAX));

    }

   /* 
    *  phi_rot+=0.15;
    *  if (phi_rot>=2*M_PI) phi_rot=0;
    *  sphere[0]->moveTo(cos(phi_rot),0.0,-sin(phi_rot));
    *  sphere[1]->moveTo(-sin(phi_rot),cos(phi_rot),0.0);
    *  sphere[2]->moveTo(0,   -sin(phi_rot),cos(phi_rot));
    */
  updateGL();
} 
