/*  kglobject.cpp - 
    This file is part of KGL
    Copyright (C) 2000  Antonio Larrosa Jimenez
    KGL' homepage : http://www.arrakis.es/~rlarrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kglobject.h"
#include <qgl.h>

KGLObject::KGLObject(GLfloat x, GLfloat y, GLfloat z)
{
  m_pos[0]=x;
  m_pos[1]=y;
  m_pos[2]=z;
  m_rotationAxis[0]=0;
  m_rotationAxis[1]=0;
  m_rotationAxis[2]=0;
  m_rotationAngle=0;
  m_list=0;
  m_color[0]=m_defaultcolor[0];
  m_color[1]=m_defaultcolor[1];
  m_color[2]=m_defaultcolor[2];
  m_color[3]=m_defaultcolor[3];
  visible=true;
}

KGLObject::~KGLObject()
{
  glDeleteLists(m_list,1);
}

void KGLObject::paint()
{
  if (!visible) return;
  
  glTranslatef(m_pos[0],m_pos[1],m_pos[2]);
  if (m_rotationAngle!=0) glRotatef(m_rotationAngle,
      m_rotationAxis[0],m_rotationAxis[1],m_rotationAxis[2]);
  glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, m_color );
  glCallList(m_list);
  if (m_rotationAngle!=0) glRotatef(-m_rotationAngle,
      m_rotationAxis[0],m_rotationAxis[1],m_rotationAxis[2]);
  glTranslatef(-m_pos[0],-m_pos[1],-m_pos[2]);
}

void KGLObject::color(const QColor &c, GLfloat alpha)
{
  m_defaultcolor[0]=((GLfloat)c.red())/255.0;
  m_defaultcolor[1]=((GLfloat)c.green())/255.0;
  m_defaultcolor[2]=((GLfloat)c.blue())/255.0;
  m_defaultcolor[3]=alpha;
}
GLfloat KGLObject::m_defaultcolor[4]={1.0,1.0,1.0,1.0};
GLfloat KGLObject::m_tempcolor[4]={1.0,1.0,1.0,1.0};

GLfloat *KGLObject::glcolor(const QColor &c, GLfloat alpha)
{
  m_tempcolor[0]=((GLfloat)c.red())/255.0;
  m_tempcolor[1]=((GLfloat)c.green())/255.0;
  m_tempcolor[2]=((GLfloat)c.blue())/255.0;
  m_tempcolor[3]=alpha;
  return m_tempcolor;
}

void KGLObject::moveTo(GLfloat x, GLfloat y, GLfloat z)
{
  m_pos[0]=x;
  m_pos[1]=y;
  m_pos[2]=z;
}  

void KGLObject::rotate(GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
{
  m_rotationAxis[0]=x;
  m_rotationAxis[1]=y;
  m_rotationAxis[2]=z;
  m_rotationAngle=angle;
}
  
void KGLObject::setColor(const QColor &c)
{
  glcolor(c);
  m_color[0]=m_tempcolor[0];
  m_color[1]=m_tempcolor[1];
  m_color[2]=m_tempcolor[2];
  m_color[3]=m_tempcolor[3];
}
