/*  kgluniverse.cpp - 
    This file is part of KGL
    Copyright (C) 2000  Antonio Larrosa Jimenez
    KGL' homepage : http://www.arrakis.es/~rlarrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kgluniverse.h"
#include <kaction.h>
#include <klocale.h>
#include <kdebug.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qpopupmenu.h>
#include <math.h>
#include <stdio.h>
#include <GL/glu.h>
#include "kglcylinder.h"
#include "kgltriangle.h"
#include "kglsphere.h"
#include "kglrectangle.h"
#include "kglarrow.h"

KGLUniverse::KGLUniverse( QWidget* parent, const char* name ) : QGLWidget (parent,name)
{
  m_isInitialized=false;
  m_paintElements=c_Axis;
  for (int i=0;i<c_NumObservers;i++)
  {
    m_currentObserver=i;
    m_pos[m_currentObserver][0]=0;
    m_pos[m_currentObserver][1]=0;
    m_pos[m_currentObserver][2]=0;
    m_viewAngle[m_currentObserver]=40.0;
    m_distanceToCenter[m_currentObserver]=0;
    
    lookTo(0.0,0.0,0.0);

    m_dir[m_currentObserver][0]=dir[0];
    m_dir[m_currentObserver][1]=dir[1];
    m_dir[m_currentObserver][2]=dir[2];
    m_obs[m_currentObserver][0]=obs[0];
    m_obs[m_currentObserver][1]=obs[1];
    m_obs[m_currentObserver][2]=obs[2];
  }

  pos[0]=0;
  pos[1]=0;
  pos[2]=0;
  viewAngle=40.0;
  distanceToCenter=0;

  m_currentObserver=0;
  m_staticObserver=false;
  lookTo(0.0,0.0,0.0);

  disableUpdates=false;
  prevTwist=0;
//  dir[0]=-1.0;
//  dir[1]=0.0;
//  dir[2]=0.0;
//  obs[0]=0.0;
//  obs[1]=0.0;
//  obs[2]=1.0;
//  delta[0]=delta[1]=delta[2]=0.0;
  objects=new QList<KGLObject>();
  mouseLMBAction=ac_rotateLookDirection;
  mouseMMBAction=ac_rotatePointOfView;

  m_delay= new QTimer(this);
  connect(m_delay,SIGNAL(timeout()), this, SLOT(hideRotationCenter()));

  /*
  m_timer=new QTimer(this);
  connect(m_timer,SIGNAL(timeout()), this, SLOT(timerTriggered()));
  */
}

KGLUniverse::~KGLUniverse()
{
// XXX delete everything !
  objects->setAutoDelete(true);
  delete objects;
}

 
void KGLUniverse::initializeGL()
{
  // Set up the lights

  GLfloat whiteAmb[4] = {1.0, 1.0, 1.0, 1.0};
  GLfloat white[4] = {2.0, 2.0, 2.0, 1.0};
  GLfloat lightPos[4] = {30.0, 30.0, 30.0, 1.0};
/*  GLfloat white[4] = {1.5, 1.5, 1.5, 1.0};
  GLfloat lightPos[4] = {1.0, 1.0, 1.0, 1.0};
  */

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, whiteAmb);

  glMaterialfv(GL_FRONT, GL_DIFFUSE, white);
  glMaterialfv(GL_FRONT, GL_SPECULAR, white);
  glMaterialf(GL_FRONT, GL_SHININESS, 20.0);

  glLightfv(GL_LIGHT0, GL_DIFFUSE, white);    // enable diffuse
  glLightfv(GL_LIGHT0, GL_SPECULAR, white);   // enable specular
  glLightfv(GL_LIGHT0, GL_POSITION, lightPos); 
//  glEnable(GL_FOG);

  QImage img1( "kedit.xpm" );

  KGLObject::color(Qt::white);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  glEnable(GL_TEXTURE_2D);  
  glShadeModel(GL_SMOOTH);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_DEPTH_TEST);

  glClearColor( 0.0, 0.0, 0.0, 0.0 );
  glEnable(GL_CULL_FACE); 
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );   

  makeObject( img1 );

/*  KGLObject::color(Qt::red);
  axis[0]=new KGLCylinder(0.0,0.0,0.0, 1.0,0.0,0.0, 0.05, 0.05);
  KGLObject::color(Qt::green);
  axis[1]=new KGLCylinder(0.0,0.0,0.0, 0.0,1.0,0.0, 0.05, 0.05);
  KGLObject::color(Qt::blue);
  axis[2]=new KGLCylinder(0.0,0.0,0.0, 0.0,0.0,1.0, 0.05, 0.05);
  */

  KGLObject::color(Qt::red);
  axis[0]=new KGLArrow(0.0,0.0,0.0, 1.0,0.0,0.0, 0.05, 0.1);
  axis[3]=new KGLArrow(-1.0,0.0,0.0, 0.0,0.0,0.0, 0.02, 0.05);
  KGLObject::color(Qt::green);
  axis[1]=new KGLArrow(0.0,0.0,0.0, 0.0,1.0,0.0, 0.05, 0.1);
  axis[4]=new KGLArrow(0.0,-1.0,0.0, 0.0,0.0,0.0, 0.02, 0.05);
  KGLObject::color(Qt::blue);
  axis[2]=new KGLArrow(0.0,0.0,0.0, 0.0,0.0,1.0, 0.05, 0.1);
  axis[5]=new KGLArrow(0.0,0.0,-1.0, 0.0,0.0,0.0, 0.02, 0.05);
  
  KGLObject::color(Qt::cyan);
  rotationCenter=new KGLSphere(0.0,0.0,0.0, 0.2);
//  rotationCenter->hide();

  //  KGLObject::color(Qt::white);
//  axis[1]=new KGLCylinder(0, 0.0,0.0,0.0, 0.0,1.57, 0.05, 0.05, 1.0);

//  m_timer->start(150);
  m_isInitialized=true;
}

void KGLUniverse::paintObserver(GLfloat *p, GLfloat *d, GLfloat *o)
{
  KGLArrow a1(p[0],p[1],p[2],
      p[0]+d[0],p[1]+d[1],p[2]+d[2],
      0.05, 0.1);
  a1.setColor(Qt::magenta);
  KGLArrow a2(p[0],p[1],p[2],
      p[0]+o[0],p[1]+o[1],p[2]+o[2],
      0.03, 0.06);
  a2.setColor(Qt::yellow);
  a1.paint();
  a2.paint();
}

void KGLUniverse::paintGL()
{
// glDisable(GL_DEPTH_TEST);   
// glClear( GL_COLOR_BUFFER_BIT );
//printf("Universe::paint\n");
  if (disableUpdates) return;
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);        
 glPushMatrix();
    glLoadIdentity();
 // glScalef(5.0,5.0,5.0);
  glEnable(GL_DEPTH_TEST);
// glEnable(GL_FOG);

//  printf("pos =    %g %g %g\n",pos[0],pos[1],pos[2]);
//  printf("lookTo = %g %g %g \t\t (%g\t%g)\n",pos[0]+dir[0],pos[1]+dir[1],pos[2]+dir[2],phi,psi);

  if ((m_staticObserver)&&(m_currentObserver!=m_currentStaticObserver))
  {  
    unsigned int co=m_currentStaticObserver;
    gluLookAt(m_pos[co][0],m_pos[co][1],m_pos[co][2],
	m_pos[co][0]+m_dir[co][0],m_pos[co][1]+m_dir[co][1],m_pos[co][2]+m_dir[co][2],
	m_obs[co][0],m_obs[co][1],m_obs[co][2]);
  }
  else
    gluLookAt(pos[0],pos[1],pos[2],
	pos[0]+dir[0],pos[1]+dir[1],pos[2]+dir[2],
	obs[0],obs[1],obs[2]);

  if (m_paintElements & c_Axis)
  {
    axis[0]->paint();
    axis[1]->paint();
    axis[2]->paint();
    axis[3]->paint();
    axis[4]->paint();
    axis[5]->paint();
  }

  rotationCenter->paint();

  if (m_paintElements & c_Obs1)
  {
    if (m_currentObserver==0) paintObserver(pos,dir,obs);
    else paintObserver(m_pos[0],m_dir[0],m_obs[0]);
  }
  if (m_paintElements & c_Obs2)
  {
    if (m_currentObserver==1) paintObserver(pos,dir,obs);
    else paintObserver(m_pos[1],m_dir[1],m_obs[1]);
  }
  if (m_paintElements & c_Obs3)
  {
    if (m_currentObserver==2) paintObserver(pos,dir,obs);
    else paintObserver(m_pos[2],m_dir[2],m_obs[2]);
  }

  /*
 sphere[0]->paint();
 sphere[1]->paint();
 sphere[2]->paint();
*/
 
 for (KGLObject *it=objects->first();it!=0L;it=objects->next())
   it->paint();

/*for (int i=0;i<400;i++) 
 {
	if (((i+1)%15)!=0) glTranslatef(0.0, 4.0, 0.0);
	else  glTranslatef(5.0, -4*15.0, 0.0);
        glCallList( object[0] );
  };
*/
// glCallList( object[0] );
// glPopMatrix();  
// glPushMatrix();
// glCallList( coord[0] );
// glPopMatrix();  
// glPushMatrix();
// glCallList( coord[1] );
//glEnable(GL_DEPTH_TEST);   
// glPopMatrix();  
// glPushMatrix();
// glCallList( coord[2] );
// glPopMatrix();  
// glPushMatrix();
// glCallList( coord[3] );
/* glBegin(GL_TRIANGLES); 
static GLfloat blue[] = { 0.0, 1.0, 1.0, 1.0 };
static GLfloat red[] = { 1.0, 0.0, 0.0, 1.0 };
glColor3f(0,0,1);
glVertex3f( 5.0, 0.0, 0.0);
        glVertex3f( 0.0,  5.0, 0.0);
        glVertex3f( 0.0,  0.0, 5.0); 
glColor3f(1,0,0);
        glVertex3f( 0.0,  5.0, 0.0);
glVertex3f( 5.0, 0.0, 0.0);
        glVertex3f( 0.0,  0.0, 5.0); 
 glEnd(); */

  QPixmap pixmap(200,18);
  pixmap.fill(Qt::black);
  QPainter p;
  p.begin(&pixmap);
  p.setPen(Qt::white);
  p.setBrush(Qt::white);
  QString s=QString("x: %1, y: %2, z: %3").arg(pos[0]).arg(pos[1]).arg(pos[2]);
  p.drawText(2,16,s);
  p.end();

  QImage Image=pixmap.convertToImage().mirror(false,true);
//  Image.invertPixels();
//  glRasterPos2i(1, 1);
 glDrawPixels(200, 18, GL_RGBA, GL_UNSIGNED_BYTE, Image.bits());
      
 glPopMatrix();  
// glFlush();
}

/*
void KGLUniverse::paintEvent(QPaintEvent *ev)
{
  QGLWidget::paintEvent(ev);

  
//  QPixmap pixmap(80,60);
 * QPainter p;
  p.begin(&pixmap);
  QString s=QString("x: %1, y: %2, z: %3").arg(pos[0]).arg(pos[1]).arg(pos[2]);
  p.drawText(2,2,s);
  p.end();

  p.begin(this);
  p.drawPixmap(width()-85,-65,pixmap);
  p.end();
* //  
}*/

void KGLUniverse::resizeGL( int w, int h )
{
    glViewport( 0, 0, w, h );
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
//    glFrustum( -1.0, 1.0, -1.0, 1.0, frustumNear, 800.0 );
//    glFrustum( -2.0, 2.0, -2.0, 2.0, frustumNear, 800.0 );
    gluPerspective(viewAngle,0.75,0.4,800.0);
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
//    glTranslatef( 0.0, 0.0, -70.0 );    
}
 
GLuint KGLUniverse::makeObject( const QImage& )
{
  /*
  QImage img1( "qtlogo.bmp" );
  QImage img2( "kedit.xpm" );

    KGLObject::color(QColor(100,0,100),0);
    sphere[0]=new KGLSphere(1,0,0, 0.2, 24);
    KGLObject::color(QColor(100,100,0),0);
    sphere[1]=new KGLSphere(0,1,0, 0.2, 24);
    KGLObject::color(QColor(0,100,100),0);
    sphere[2]=new KGLSphere(0,0,1, 0.2, 24);
*/
    
    return 0;     
}
                                 

/*void KGLUniverse::moveView(int v)
{
  GLfloat center[3];
  center[0]=12;
  center[1]=8;
  center[2]=0;
  GLfloat p[3];
  p[0]=0;
  p[1]=8;
  p[2]=0;
  GLfloat c[3];
  c[0]=center[0]-p[0];
  c[1]=center[1]-p[1];
  c[2]=center[2]-p[2];

  GLfloat r=sqrt(c[0]*c[0]+c[1]*c[1]+c[2]*c[2]);
  GLfloat phi=((c[0]==0) ? 0 : atan(c[0]/c[1]));

  GLfloat psi=2*M_PI*v/360;

  printf("%g\t%g\t%d\n",phi,psi,v);
 
  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);

  pos[0]=center[0]-r*dir[0];
  pos[1]=center[1]-r*dir[1];
  pos[2]=center[2]-r*dir[2];

  obs[0]=-sin(psi)*sin(phi);
  obs[1]=-sin(psi)*cos(phi);
  obs[2]=cos(psi);

  printf("pos : %g\t%g\t%g\n",pos[0],pos[1],pos[2]);
  printf("dir : %g\t%g\t%g\n",dir[0],dir[1],dir[2]);

  updateGL();   

}
*/

void KGLUniverse::lookTo(GLfloat x, GLfloat y, GLfloat z)
{
  GLfloat r=sqrt((pos[0]-x)*(pos[0]-x)+(pos[1]-y)*(pos[1]-y)+(pos[2]-z)*(pos[2]-z));
  phi=(((x-pos[0])==0) ? 0 : atan2((x-pos[0]),(y-pos[1])));

//  GLfloat psi=2*M_PI*v/360;

  psi=(((z-pos[2])==0) ? 0 : atan((z-pos[2])/sqrt((pos[1]-y)*(pos[1]-y)+(pos[0]-x)*(pos[0]-x))));
    
  printf("::lookTo %g\t%g\t%g\n",pos[0],pos[1],pos[2]);
  printf("::lookTo %g\t%g\n",phi,psi);

  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);

  obs[0]=-sin(psi)*sin(phi);
  obs[1]=-sin(psi)*cos(phi);
  obs[2]=cos(psi);
  
  distanceToCenter=r;
  
  printf("pos : %g\t%g\t%g\n",pos[0],pos[1],pos[2]);
  printf("dir : %g\t%g\t%g\n",dir[0],dir[1],dir[2]);
  printf("obs : %g\t%g\t%g\n",obs[0],obs[1],obs[2]);
  
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;
}

void KGLUniverse::moveX(int v)
{
  pos[0]=v/1000.0;
  updateGL();   
}

void KGLUniverse::moveY(int v)
{
  pos[1]=v/1000.0;
  updateGL();   
}

void KGLUniverse::moveZ(int v)
{
  pos[2]=v/1000.0;
  updateGL();   
}

void KGLUniverse::movePhi(int v)
{
  phi=v*2*M_PI/360000.0;
  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);
  updateGL();   
}

void KGLUniverse::movePsi(int v)
{
  psi=v*2*M_PI/360000.0;
  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);
  updateGL();   
}

void KGLUniverse::resetTwist()
{
  prevTwist=0;

  emit twistChanged(0);
}

void KGLUniverse::twist(int v)
{
  if (v==0) return;

  GLfloat t=v*2*M_PI/360000.0-prevTwist;
  printf("TWIST %g      %g\n",t,prevTwist);
  prevTwist+=t;

  // Now we rotate obs over the axis given by dir, v rad.

  GLfloat q0=sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2])/tan(t/2);
  // b=q0*obs + (obs x dir)
  GLfloat b[3];
  b[0]=q0*obs[0]+obs[1]*dir[2]-dir[1]*obs[2];
  b[1]=q0*obs[1]+dir[0]*obs[2]-obs[0]*dir[2];
  b[2]=q0*obs[2]+obs[0]*dir[1]-dir[0]*obs[1];
  // a= -obs.dir
  GLfloat a=-(obs[0]*dir[0]+obs[1]*dir[1]+obs[2]*dir[2]);

  // r= ( q0 * b ) + ( a * -dir) + ( -dir x b )
  obs[0]=q0*b[0]-a*dir[0]+(-dir[1])*b[2]-b[1]*(-dir[2]);
  obs[1]=q0*b[1]-a*dir[1]+b[0]*(-dir[2])-(-dir[0])*b[2];
  obs[2]=q0*b[2]-a*dir[2]+(-dir[0])*b[1]-b[0]*(-dir[1]);

  normalizeVector(obs);

  updateGL();   

}

void KGLUniverse::setViewAngle(int v)
{
  viewAngle=v/1000.0;

  printf("view Angle %g\n",viewAngle);
//  glPushMatrix();
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
//  glFrustum( -1.0, 1.0, -1.0, 1.0, frustumNear, 800.0 );
//  glOrtho(-1.0, 1.0, -1.0, 1.0, -10.0, 800.0 );
  gluPerspective(viewAngle,0.75,0.4,800.0);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
//  glPopMatrix();

  updateGL(); 
}

void KGLUniverse::setDistanceToCenter(int v)
{
  if (v<=0) return;
  m_delay->stop();
  distanceToCenter=v/1000.0;
  rotationCenter->moveTo(pos[0]+distanceToCenter*dir[0],
      pos[1]+distanceToCenter*dir[1],pos[2]+distanceToCenter*dir[2]);
  rotationCenter->show();
//  QTimer::singleShot(1800,this,SLOT(hideRotationCenter()));
  m_delay->start(1800,true);
  updateGL();
}

void KGLUniverse::hideRotationCenter()
{
  rotationCenter->hide();
  updateGL();
}

void KGLUniverse::moveForward()
{
  GLfloat speed=0.8;
  pos[0]+=speed*dir[0];
  pos[1]+=speed*dir[1];
  pos[2]+=speed*dir[2];

  disableUpdates=true;
  emitCoords();
  disableUpdates=false;

  updateGL();
}

void KGLUniverse::lookToOrigin()
{
  lookTo(0.0,0.0,0.0);
}

void KGLUniverse::moveBackward()
{
  GLfloat speed=0.8;
  pos[0]-=speed*dir[0];
  pos[1]-=speed*dir[1];
  pos[2]-=speed*dir[2];
  
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;

  updateGL();
}

/*
void KGLUniverse::timerTriggered()
{
//  for (int i=0;i<traces;i++)
//  {
    m_animPoint++;
//    if (m_animPoint[i]==qn[i]) m_animPoint[i]=0;
//  }
    phi_rot+=0.15;
    if (phi_rot>=2*M_PI) phi_rot=0;
  sphere[0]->moveTo(cos(phi_rot),0.0,-sin(phi_rot));
  sphere[1]->moveTo(-sin(phi_rot),cos(phi_rot),0.0);
  sphere[2]->moveTo(0,   -sin(phi_rot),cos(phi_rot));

  updateGL();
};
*/

void KGLUniverse::emitCoords()
{
  emit movedX(pos[0]*1000);
  emit movedY(pos[1]*1000);
  emit movedZ(pos[2]*1000);
//  emit phiChanged(phi*360000.0/(2.0*M_PI));
//  emit psiChanged(psi*360000.0/(2.0*M_PI));
  emit distanceToCenterChanged(distanceToCenter*1000);
  
}

void KGLUniverse::moveViewPoint(GLfloat x,GLfloat y,GLfloat z)
{
  pos[0]=x;
  pos[1]=y;
  pos[2]=z;
  updateGL();
  emitCoords();
}

void KGLUniverse::mousePressEvent(QMouseEvent *ev)
{
  /*
  dphi=((width()/2.0)-ev->x())*M_PI/(10*width()/2.0);
  dpsi=(ev->y()-(height()/2.0))*M_PI/(10*height()/2.0);

  dir[0]=cos(psi+dpsi)*sin(phi+dphi);
  dir[1]=cos(psi+dpsi)*cos(phi+dphi);
  dir[2]=sin(psi+dpsi);

  updateGL();
  */
  if (ev->button() == MidButton)
  {
    mouseRotCenterX=ev->x();
    mouseRotCenterY=ev->y();
    mouseRotPhi=phi;
    mouseRotPsi=psi;
    mouseState=mouseMMBAction;
  }
  else if (ev->button() == LeftButton)
  {
    mouseRotCenterX=ev->x();
    mouseRotCenterY=ev->y();
    mouseRotPhi=phi;
    mouseRotPsi=psi;
    mouseState=mouseLMBAction;
  }
  else if (ev->button() == RightButton)
  {
    QPopupMenu *options=new QPopupMenu(this);
    KSelectAction *modesLMB=new KSelectAction(i18n("Left Mouse Button"),0,options);
    KSelectAction *modesMMB=new KSelectAction(i18n("Middle Mouse Button"),0,options);
    QStringList strlist;
    strlist.append(i18n("Move X"));
    strlist.append(i18n("Move Y"));
    strlist.append(i18n("Move Z"));
    strlist.append(i18n("Twist"));
    strlist.append(i18n("Rotate Looking Direction"));
    strlist.append(i18n("Rotate Point of View"));
    strlist.append(i18n("Move Distance to Center of Rotation"));
    modesLMB->setItems(strlist);
    modesMMB->setItems(strlist);
    
    connect(modesLMB, SIGNAL(activated(int)), this, SLOT(setLMBAction(int)));
    connect(modesMMB, SIGNAL(activated(int)), this, SLOT(setMMBAction(int)));

    modesLMB->plug(options);
    modesLMB->setCurrentItem(static_cast<int>(mouseLMBAction));
    modesMMB->plug(options);
    modesMMB->setCurrentItem(static_cast<int>(mouseMMBAction));

    options->insertSeparator();

    KSelectAction *changeobserver=new KSelectAction(i18n("Observers"),0,options);
    strlist.clear();
    for (int i=0;i<c_NumObservers;i++) strlist << QString::number(i+1);
    changeobserver->setItems(strlist);

    connect(changeobserver, SIGNAL(activated(int)), this, SLOT(changeObserver(int)));

    changeobserver->plug(options);
    changeobserver->setCurrentItem(m_currentObserver);

    KToggleAction *staticobserver=new KToggleAction(i18n("Static Observer"), 0/*BarIcon("undo")*/, 0,
	options, "staticObserver");
    connect(staticobserver, SIGNAL(toggled(bool)), this, SLOT(setStaticObserver(bool)));
    staticobserver->setEnabled(true);
    staticobserver->plug(options);
    staticobserver->setChecked(m_staticObserver);

    /*QPopupMenu *drawOptions=new QPopupMenu(options);
    
    
    KToggleAction *draw=new KToggleAction(i18n("Draw Axis"), 0, 0,
       	drawOptions, "drawAxis");
    draw->setEnabled(true);
    draw->plug(drawOptions);
    draw->setChecked(m_paintElements & c_Axis );
    
    draw=new KToggleAction(i18n("Draw Observer 1"), 0, 0,
       	drawOptions, "drawObserver1");
    draw->setEnabled(true);
    draw->plug(drawOptions);
    draw->setChecked(m_paintElements & c_Obs1 );

    draw=new KToggleAction(i18n("Draw Observer 2"), 0, 0,
       	drawOptions, "drawObserver2");
    draw->setEnabled(true);
    draw->plug(drawOptions);
    draw->setChecked(m_paintElements & c_Obs2 );
 
    draw=new KToggleAction(i18n("Draw Observer 3"), 0, 0,
       	drawOptions, "drawObserver3");
    draw->setEnabled(true);
    draw->plug(drawOptions);
    draw->setChecked(m_paintElements & c_Obs3 );
    connect(drawOptions, SIGNAL(activated(int)), this, SLOT(toggleDrawElements(int)));

    options->insertItem(i18n("Display Options"),drawOptions);
    */
    KSelectAction *drawoptions=new KSelectAction(i18n("Draw Options"),0,options);
    strlist.clear();
    strlist << i18n("Toggle Axis");
    for (int i=0;i<c_NumObservers;i++) strlist << i18n("Toggle Observer %1").arg(i+1);
    drawoptions->setItems(strlist);

    connect(drawoptions, SIGNAL(activated(int)), this, SLOT(toggleDrawElements(int)));

    drawoptions->plug(options);
//    drawoptions->setCurrentItem(m_currentObserver);
    
    options->exec(mapToGlobal(ev->pos()));
  }
}

void KGLUniverse::setLMBAction(int a)
{
  mouseLMBAction=static_cast<mouseActions>(a);
}

void KGLUniverse::setMMBAction(int a)
{
  mouseMMBAction=static_cast<mouseActions>(a);
}

void KGLUniverse::normalizeVector(GLfloat *v)
{
  GLfloat norm=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);

  v[0]/=norm;
  v[1]/=norm;
  v[2]/=norm;
  
}

void KGLUniverse::multiplyVector(GLfloat *v, GLfloat m)
{
  v[0]*=m;
  v[1]*=m;
  v[2]*=m;
}

void KGLUniverse::rotatePointOfView(GLfloat x, GLfloat y)
{
  GLfloat p[3];
  GLfloat norm=sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2]);

  p[0]=-dir[0]/norm;
  p[1]=-dir[1]/norm;
  p[2]=-dir[2]/norm;

  // p is now the point in the unit Sphere which we want to rotate

  // obs defines an orientation which is "up" on ev->y() (obs==e2)

  GLfloat e2[3];
  e2[0]=obs[0];
  e2[1]=obs[1];
  e2[2]=obs[2];

  // e1 is  e2 x p , that is, the "right" direction on ev->x()
  GLfloat e1[3];
  e1[0]=e2[1]*p[2]-p[1]*e2[2];
  e1[1]=p[0]*e2[2]-e2[0]*p[2];
  e1[2]=e2[0]*p[1]-p[0]*e2[1];

  printf("e1 (%g %g %g) \n",e1[0],e1[1],e1[2]);
  printf("e2 (%g %g %g) \n",e2[0],e2[1],e2[2]);
  normalizeVector(e1);
  normalizeVector(e2);

  printf("e1 %g %g %g \n",e1[0],e1[1],e1[2]);
  printf("e2 %g %g %g \n",e2[0],e2[1],e2[2]);

  printf("x %g  \n",x);
  printf("y %g  \n",y);

  multiplyVector(e1,x);
  multiplyVector(e2,y);

  printf("e1,,%g %g %g \n",e1[0],e1[1],e1[2]);
  printf("e2,,%g %g %g \n",e2[0],e2[1],e2[2]);

  GLfloat r[3];
  r[0]=e1[0]+e2[0];
  r[1]=e1[1]+e2[1];
  r[2]=e1[2]+e2[2];
  printf("r %g %g %g \n",r[0],r[1],r[2]);

  // v= p x r
  GLfloat v[3];
  v[0]=p[1]*r[2]-r[1]*p[2];
  v[1]=r[0]*p[2]-p[0]*r[2];
  v[2]=p[0]*r[1]-r[0]*p[1];

  printf("v %g %g %g \n",v[0],v[1],v[2]);

  // v is now the axis around which we want to rotate a distance given by 
  // the (Manhattan ?) norm of ( ev->x() ev->y() )

  GLfloat xrad=fabs(x*M_PI/width());
  GLfloat yrad=fabs(y*M_PI/height());
  GLfloat alpha= QMAX(xrad, yrad);

  printf("alpha %g (xrad %g , yrad %g)\n",alpha,xrad,yrad);
  // Now we rotate p over the axis given by v, alpha rad.

  GLfloat q0=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])/tan(alpha/2);
  // b=q0*p + (p x v) 
  GLfloat b[3];
  b[0]=q0*p[0]+p[1]*v[2]-v[1]*p[2];
  b[1]=q0*p[1]+v[0]*p[2]-p[0]*v[2];
  b[2]=q0*p[2]+p[0]*v[1]-v[0]*p[1];
  // a= -p.v
  GLfloat a=-(p[0]*v[0]+p[1]*v[1]+p[2]*v[2]);

  // r= ( q0 * b ) + ( a * -v) + ( -v x b )
  r[0]=q0*b[0]-a*v[0]+(-v[1])*b[2]-b[1]*(-v[2]);
  r[1]=q0*b[1]-a*v[1]+b[0]*(-v[2])-(-v[0])*b[2];
  r[2]=q0*b[2]-a*v[2]+(-v[0])*b[1]-b[0]*(-v[1]);

  normalizeVector(r);

  printf("r %g %g %g\n",r[0],r[1],r[2]);

  // We have in r the rotated p point (over the unit sphere).

  pos[0]=pos[0]+distanceToCenter*(dir[0]+r[0]);
  pos[1]=pos[1]+distanceToCenter*(dir[1]+r[1]);
  pos[2]=pos[2]+distanceToCenter*(dir[2]+r[2]);

  dir[0]=-r[0];
  dir[1]=-r[1];
  dir[2]=-r[2];

  // We are almost finished, we only have to update obs to be
  // orthogonal to dir

  // We're doing (dir x obs) x dir to get a vector v which will
  // be in the "direction" of obs (in fact, it's not exactly on the
  // same direction, but in the plane generated by obs and dir, and
  // orthogonal to dir)

  b[0]=dir[1]*obs[2]-obs[1]*dir[2];
  b[1]=obs[0]*dir[2]-dir[0]*obs[2];
  b[2]=dir[0]*obs[1]-obs[0]*dir[1];

  v[0]=b[1]*dir[2]-dir[1]*b[2];
  v[1]=dir[0]*b[2]-b[0]*dir[2];
  v[2]=b[0]*dir[1]-dir[0]*b[1];

  obs[0]=v[0];
  obs[1]=v[1];
  obs[2]=v[2];

  normalizeVector(obs);

  phi=( ( dir[0]==0 ) ? 0 : atan2(dir[0],dir[1]) );
  psi=( ( dir[2]==0 ) ? 0 : atan( dir[2]/sqrt(dir[1]*dir[1]+dir[0]*dir[0])) );


  //    updateGL();
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;
  updateGL();


}

void KGLUniverse::rotateLookDirection(GLfloat x,GLfloat y)
{
  GLfloat rad=viewAngle*M_PI/360.0;
  //  GLfloat dphi=(mouseRotCenterX-ev->x())*M_PI/((viewAngle/4.0)*width()/2.0);
  //  GLfloat dpsi=(ev->y()-mouseRotCenterY)*M_PI/((viewAngle/4.0)*height()/2.0);

  //
  //   GLfloat dphi=(mouseRotCenterX-ev->x())*rad/(width()/2.0);
  //   GLfloat dpsi=(ev->y()-mouseRotCenterY)*rad/(height()/2.0);
  //   phi=mouseRotPhi+dphi;
  //   psi=mouseRotPsi+dpsi;

  //   dir[0]=cos(psi)*sin(phi);
  //   dir[1]=cos(psi)*cos(phi);
  //   dir[2]=sin(psi);
   
  GLfloat p[3];
  GLfloat norm=sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2]);

  p[0]=dir[0]/norm;
  p[1]=dir[1]/norm;
  p[2]=dir[2]/norm;

  // p is now the point in the unit Sphere which we are looking to

  // obs defines an orientation which is "up" on ev->y() (obs==e2)

  GLfloat e2[3];
  e2[0]=obs[0];
  e2[1]=obs[1];
  e2[2]=obs[2];

  // e1 is p x e2 , that is, the "right" direction on ev->x()
  GLfloat e1[3];
  e1[0]=-e2[1]*p[2]+p[1]*e2[2];
  e1[1]=-p[0]*e2[2]+e2[0]*p[2];
  e1[2]=-e2[0]*p[1]+p[0]*e2[1];

  printf("e1 (%g %g %g) \n",e1[0],e1[1],e1[2]);
  printf("e2 (%g %g %g) \n",e2[0],e2[1],e2[2]);
  normalizeVector(e1);
  normalizeVector(e2);

  printf("e1 %g %g %g \n",e1[0],e1[1],e1[2]);
  printf("e2 %g %g %g \n",e2[0],e2[1],e2[2]);

  printf("x %g  \n",x);
  printf("y %g  \n",y);

  multiplyVector(e1,x);
  multiplyVector(e2,y);

  printf("e1,,%g %g %g \n",e1[0],e1[1],e1[2]);
  printf("e2,,%g %g %g \n",e2[0],e2[1],e2[2]);

  GLfloat r[3];
  r[0]=e1[0]+e2[0];
  r[1]=e1[1]+e2[1];
  r[2]=e1[2]+e2[2];
  printf("r %g %g %g \n",r[0],r[1],r[2]);

  // v= p x r
  GLfloat v[3];
  v[0]=p[1]*r[2]-r[1]*p[2];
  v[1]=r[0]*p[2]-p[0]*r[2];
  v[2]=p[0]*r[1]-r[0]*p[1];

  printf("v %g %g %g \n",v[0],v[1],v[2]);

  // v is now the axis around which we want to rotate a distance given by 
  // the (Manhattan ?) norm of ( ev->x() ev->y() )

  GLfloat xrad=fabs(x*rad/(width()/2.0));
  GLfloat yrad=fabs(y*rad/(height()/2.0));
  GLfloat alpha= QMAX(xrad, yrad);

  printf("alpha %g (xrad %g , yrad %g)\n",alpha,xrad,yrad);
  // Now we rotate p over the axis given by v, alpha rad.

  GLfloat q0=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])/tan(alpha/2);
  // b=q0*p + (p x v) 
  GLfloat b[3];
  b[0]=q0*p[0]+p[1]*v[2]-v[1]*p[2];
  b[1]=q0*p[1]+v[0]*p[2]-p[0]*v[2];
  b[2]=q0*p[2]+p[0]*v[1]-v[0]*p[1];
  // a= -p.v
  GLfloat a=-(p[0]*v[0]+p[1]*v[1]+p[2]*v[2]);

  // r= ( q0 * b ) + ( a * -v) + ( -v x b )
  r[0]=q0*b[0]-a*v[0]+(-v[1])*b[2]-b[1]*(-v[2]);
  r[1]=q0*b[1]-a*v[1]+b[0]*(-v[2])-(-v[0])*b[2];
  r[2]=q0*b[2]-a*v[2]+(-v[0])*b[1]-b[0]*(-v[1]);

  normalizeVector(r);

  printf("r %g %g %g\n",r[0],r[1],r[2]);

  // We have in r the rotated p point (over the unit sphere).

  dir[0]=r[0];
  dir[1]=r[1];
  dir[2]=r[2];

  // We are almost finished, we only have to update obs to be
  // orthogonal to dir

  // We're doing (dir x obs) x dir to get a vector v which will
  // be in the direction of obs.

  b[0]=dir[1]*obs[2]-obs[1]*dir[2];
  b[1]=obs[0]*dir[2]-dir[0]*obs[2];
  b[2]=dir[0]*obs[1]-obs[0]*dir[1];

  v[0]=b[1]*dir[2]-dir[1]*b[2];
  v[1]=dir[0]*b[2]-b[0]*dir[2];
  v[2]=b[0]*dir[1]-dir[0]*b[1];

  normalizeVector(v);

  obs[0]=v[0];
  obs[1]=v[1];
  obs[2]=v[2];

  phi=( ( dir[0]==0 ) ? 0 : atan2(dir[0],dir[1]) );
  psi=( ( dir[2]==0 ) ? 0 : atan( dir[2]/sqrt(dir[1]*dir[1]+dir[0]*dir[0])) );

  updateGL();

  emit phiChanged((phi)*360000.0/(2.0*M_PI));
  emit psiChanged((psi)*360000.0/(2.0*M_PI)); 
/*
  GLfloat p[3];
  GLfloat norm=sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2]);

  p[0]=-dir[0]/norm;
  p[1]=-dir[1]/norm;
  p[2]=-dir[2]/norm;

  // p is now the point in the unit Sphere which we want to rotate

  // obs defines an orientation which is "up" on ev->y() (obs==e2)

  GLfloat e2[3];
  e2[0]=obs[0];
  e2[1]=obs[1];
  e2[2]=obs[2];

  // e1 is  e2 x p , that is, the "right" direction on ev->x()
  GLfloat e1[3];
  e1[0]=e2[1]*p[2]-p[1]*e2[2];
  e1[1]=p[0]*e2[2]-e2[0]*p[2];
  e1[2]=e2[0]*p[1]-p[0]*e2[1];

  printf("e1 (%g %g %g) \n",e1[0],e1[1],e1[2]);
  printf("e2 (%g %g %g) \n",e2[0],e2[1],e2[2]);
  normalizeVector(e1);
  normalizeVector(e2);

  printf("e1 %g %g %g \n",e1[0],e1[1],e1[2]);
  printf("e2 %g %g %g \n",e2[0],e2[1],e2[2]);

  printf("x %g  \n",x);
  printf("y %g  \n",y);

  multiplyVector(e1,x);
  multiplyVector(e2,y);

  printf("e1,,%g %g %g \n",e1[0],e1[1],e1[2]);
  printf("e2,,%g %g %g \n",e2[0],e2[1],e2[2]);

  GLfloat r[3];
  r[0]=e1[0]+e2[0];
  r[1]=e1[1]+e2[1];
  r[2]=e1[2]+e2[2];
  printf("r %g %g %g \n",r[0],r[1],r[2]);

  // v= p x r
  GLfloat v[3];
  v[0]=p[1]*r[2]-r[1]*p[2];
  v[1]=r[0]*p[2]-p[0]*r[2];
  v[2]=p[0]*r[1]-r[0]*p[1];

  printf("v %g %g %g \n",v[0],v[1],v[2]);

  // v is now the axis around which we want to rotate a distance given by 
  // the (Manhattan ?) norm of ( ev->x() ev->y() )

  GLfloat xrad=fabs(x*M_PI/width());
  GLfloat yrad=fabs(y*M_PI/height());
  GLfloat alpha= QMAX(xrad, yrad);

  printf("alpha %g (xrad %g , yrad %g)\n",alpha,xrad,yrad);
  // Now we rotate p over the axis given by v, alpha rad.

  GLfloat q0=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])/tan(alpha/2);
  // b=q0*p + (p x v) 
  GLfloat b[3];
  b[0]=q0*p[0]+p[1]*v[2]-v[1]*p[2];
  b[1]=q0*p[1]+v[0]*p[2]-p[0]*v[2];
  b[2]=q0*p[2]+p[0]*v[1]-v[0]*p[1];
  // a= -p.v
  GLfloat a=-(p[0]*v[0]+p[1]*v[1]+p[2]*v[2]);

  // r= ( q0 * b ) + ( a * -v) + ( -v x b )
  r[0]=q0*b[0]-a*v[0]+(-v[1])*b[2]-b[1]*(-v[2]);
  r[1]=q0*b[1]-a*v[1]+b[0]*(-v[2])-(-v[0])*b[2];
  r[2]=q0*b[2]-a*v[2]+(-v[0])*b[1]-b[0]*(-v[1]);

  normalizeVector(r);

  printf("r %g %g %g\n",r[0],r[1],r[2]);

  // We have in r the rotated p point (over the unit sphere).

  pos[0]=pos[0]+distanceToCenter*(dir[0]+r[0]);
  pos[1]=pos[1]+distanceToCenter*(dir[1]+r[1]);
  pos[2]=pos[2]+distanceToCenter*(dir[2]+r[2]);

  dir[0]=-r[0];
  dir[1]=-r[1];
  dir[2]=-r[2];

  // We are almost finished, we only have to update obs to be
  // orthogonal to dir

  // We're doing (dir x obs) x dir to get a vector v which will
  // be in the "direction" of obs (in fact, it's not exactly on the
  // same direction, but in the plane generated by obs and dir, and
  // orthogonal to dir)

  b[0]=dir[1]*obs[2]-obs[1]*dir[2];
  b[1]=obs[0]*dir[2]-dir[0]*obs[2];
  b[2]=dir[0]*obs[1]-obs[0]*dir[1];

  v[0]=b[1]*dir[2]-dir[1]*b[2];
  v[1]=dir[0]*b[2]-b[0]*dir[2];
  v[2]=b[0]*dir[1]-dir[0]*b[1];

  obs[0]=v[0];
  obs[1]=v[1];
  obs[2]=v[2];

  phi=( ( dir[0]==0 ) ? 0 : atan2(dir[0],dir[1]) );
  psi=( ( dir[2]==0 ) ? 0 : atan( dir[2]/sqrt(dir[1]*dir[1]+dir[0]*dir[0])) );


  //    updateGL();
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;
  updateGL();
*/

}

void KGLUniverse::mouseMoveEvent(QMouseEvent *ev)
{
  switch (mouseState)
  {
    case (ac_rotatePointOfView) :
      {
	rotatePointOfView( ev->x()-mouseRotCenterX, ev->y()-mouseRotCenterY );
	mouseRotCenterX=ev->x();
	mouseRotCenterY=ev->y();
      } break;
    case (ac_rotateLookDirection) : 
      {
	rotateLookDirection( ev->x()-mouseRotCenterX, mouseRotCenterY-ev->y() );
	mouseRotCenterX=ev->x();
	mouseRotCenterY=ev->y();
      } break;
    default: kdDebug() << "Not Implemented yet" << endl;
  }
}

void KGLUniverse::mouseReleaseEvent(QMouseEvent *)
{
/*  GLfloat dphi=(mouseRotCenterX-ev->x())*M_PI/(10*width()/2.0);
  GLfloat dpsi=(ev->y()-mouseRotCenterY)*M_PI/(10*height()/2.0);
  
  psi+=dpsi;
  phi+=dphi;

  dir[0]=cos(psi)*sin(phi);
  dir[1]=cos(psi)*cos(phi);
  dir[2]=sin(psi);
  updateGL();
  emitCoords();
  */
}

void KGLUniverse::changeObserver(int i)
{
  if (m_currentObserver==static_cast<unsigned int>(i)) return;

  // Store the state of this observer

  m_pos[m_currentObserver][0]=pos[0];
  m_pos[m_currentObserver][1]=pos[1];
  m_pos[m_currentObserver][2]=pos[2];
  m_viewAngle[m_currentObserver]=viewAngle;
  m_distanceToCenter[m_currentObserver]=distanceToCenter;
  m_dir[m_currentObserver][0]=dir[0];
  m_dir[m_currentObserver][1]=dir[1];
  m_dir[m_currentObserver][2]=dir[2];
  m_obs[m_currentObserver][0]=obs[0];
  m_obs[m_currentObserver][1]=obs[1];
  m_obs[m_currentObserver][2]=obs[2];
  
  m_currentObserver=i;

  pos[0]=m_pos[m_currentObserver][0];
  pos[1]=m_pos[m_currentObserver][1];
  pos[2]=m_pos[m_currentObserver][2];
  viewAngle=m_viewAngle[m_currentObserver];
  distanceToCenter=m_distanceToCenter[m_currentObserver];
  dir[0]=m_dir[m_currentObserver][0];
  dir[1]=m_dir[m_currentObserver][1];
  dir[2]=m_dir[m_currentObserver][2];
  obs[0]=m_obs[m_currentObserver][0];
  obs[1]=m_obs[m_currentObserver][1];
  obs[2]=m_obs[m_currentObserver][2];
  
  
  disableUpdates=true;
  emitCoords();
  disableUpdates=false;
  updateGL(); 

}

void KGLUniverse::setStaticObserver(bool b)
{
  m_staticObserver=b;
  m_currentStaticObserver=m_currentObserver;
}

void KGLUniverse::toggleDrawElements(int i)
{
  printf("toggle !\n");
  int t=1<<i;
  m_paintElements=m_paintElements ^ t;
  printf("%x",m_paintElements);
}
